<?php declare(strict_types = 1);

namespace Drupal\lb_plus\EventSubscriber;

use Drupal\Core\Render\Markup;
use Drupal\twig_events\Event\TwigRenderTemplateEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

final class UiPageWrapper implements EventSubscriberInterface {

  public function onTwigRenderTemplate(TwigRenderTemplateEvent $event): void {
    $variables = $event->getVariables();
    $entity_info = $this->getEntityInfo($variables);
    if (!empty($entity_info)) {
      // Give the rendered entity an AJAX wrapper so it can be updated as
      // changes are made.
      $output = $event->getOutput();
      $wrapped_output = sprintf('<div class="lb-plus-ui-wrapper" data-toolbar-plus-entity-wrapper="%s::%s::%s">%s</div>', $entity_info['entity_type'], $entity_info['entity_id'], $entity_info['view_mode'], $output->__toString());
      $event->setOutput(Markup::create($wrapped_output));
    }
  }


  private function getEntityInfo(array $variables) {
    if (!empty($variables['elements']['#lb_plus_ui'])) {
      // Regular entities.
      return $variables['elements']['#lb_plus_ui'];
    } elseif (!empty($variables['elements']['content']['#lb_plus_ui'])) {
      // Inline blocks.
      return $variables['elements']['content']['#lb_plus_ui'];
    }
  }


  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      TwigRenderTemplateEvent::class => ['onTwigRenderTemplate'],
    ];
  }

}
